/* 
   pcapproc - PCAP procfs interface

   Copyright (C) Ho Ming Shun      cyph
                                    at	
                               cyph.ath.cx

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License version 2 as
   published by the Free Software Foundation;

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT OF THIRD PARTY RIGHTS.
   IN NO EVENT SHALL THE COPYRIGHT HOLDER(S) AND AUTHOR(S) BE LIABLE FOR ANY
   CLAIM, OR ANY SPECIAL INDIRECT OR CONSEQUENTIAL DAMAGES, OR ANY DAMAGES 
   WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN 
   ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF 
   OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

   ALL LIABILITY, INCLUDING LIABILITY FOR INFRINGEMENT OF ANY PATENTS, 
   COPYRIGHTS, TRADEMARKS OR OTHER RIGHTS, RELATING TO USE OF THIS 
   SOFTWARE IS DISCLAIMED.
*/

/*
 * Creates the following proc entries:
 * usb-regs - interesting USB registers
 * pcap - directory. each file corresponds to a PCAP bit. You can read from each file and write "1" or "0" to it.
 *
 * The included ssp_pcap.h defines absolute function addresses for SSP_PCAP_bit_set, SSP_PCAP_bit_clean and SSP_PCAP_get_bit_from_buffer. These offsets are obtained by disassembling kernel code section. AFAIK these offsets are correct for the C5.A4P firmware on E680I. Have not tried on any other phone/firmwares.
*/

#include <linux/init.h>
#include <linux/config.h>
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/proc_fs.h>
#include <asm/uaccess.h>


#include <asm/arch/hardware.h>
#include "ssp_pcap.h"

static int pcap_read_proc(char *, char **, off_t, int, int *, void *);
static int usb_reg_read_proc(char *, char **, off_t, int, int *, void *);
static int pcap_write_proc(struct file *file, const char *buffer, unsigned long count, void *data);

struct pcap_reg {
	const char* name;
	enum pcapADJBit_SetType regbit;
};

struct pcap_reg pcap_regs[] = {
	{ "SSP_PCAP_ADJ_BIT_ISR_ADCDONEI", SSP_PCAP_ADJ_BIT_ISR_ADCDONEI },
	{ "SSP_PCAP_ADJ_BIT_ISR_TSI", SSP_PCAP_ADJ_BIT_ISR_TSI },
	{ "SSP_PCAP_ADJ_BIT_ISR_1HZI", SSP_PCAP_ADJ_BIT_ISR_1HZI },
	{ "SSP_PCAP_ADJ_BIT_ISR_WHI", SSP_PCAP_ADJ_BIT_ISR_WHI },
	{ "SSP_PCAP_ADJ_BIT_ISR_WLI", SSP_PCAP_ADJ_BIT_ISR_WLI },
	{ "SSP_PCAP_ADJ_BIT_ISR_TODAI", SSP_PCAP_ADJ_BIT_ISR_TODAI },
	{ "SSP_PCAP_ADJ_BIT_ISR_USB4VI", SSP_PCAP_ADJ_BIT_ISR_USB4VI },
	{ "SSP_PCAP_ADJ_BIT_ISR_ONOFFI", SSP_PCAP_ADJ_BIT_ISR_ONOFFI },
	{ "SSP_PCAP_ADJ_BIT_ISR_ONOFF2I", SSP_PCAP_ADJ_BIT_ISR_ONOFF2I },
	{ "SSP_PCAP_ADJ_BIT_ISR_USB1VI", SSP_PCAP_ADJ_BIT_ISR_USB1VI },
	{ "SSP_PCAP_ADJ_BIT_ISR_MOBPORTI", SSP_PCAP_ADJ_BIT_ISR_MOBPORTI },
	{ "SSP_PCAP_ADJ_BIT_ISR_MB2I", SSP_PCAP_ADJ_BIT_ISR_MB2I },
	{ "SSP_PCAP_ADJ_BIT_ISR_A1I", SSP_PCAP_ADJ_BIT_ISR_A1I },
	{ "SSP_PCAP_ADJ_BIT_ISR_STI", SSP_PCAP_ADJ_BIT_ISR_STI },
	{ "SSP_PCAP_ADJ_BIT_ISR_PCI", SSP_PCAP_ADJ_BIT_ISR_PCI },
	{ "SSP_PCAP_ADJ_BIT_ISR_WARMI", SSP_PCAP_ADJ_BIT_ISR_WARMI },
	{ "SSP_PCAP_ADJ_BIT_ISR_EOLI", SSP_PCAP_ADJ_BIT_ISR_EOLI },
	{ "SSP_PCAP_ADJ_BIT_ISR_CLKI", SSP_PCAP_ADJ_BIT_ISR_CLKI },
	{ "SSP_PCAP_ADJ_BIT_ISR_SYS_RSTI", SSP_PCAP_ADJ_BIT_ISR_SYS_RSTI },
	{ "SSP_PCAP_ADJ_BIT_ISR_ADCDONE2I", SSP_PCAP_ADJ_BIT_ISR_ADCDONE2I },
	{ "SSP_PCAP_ADJ_BIT_ISR_SOFT_RESETI", SSP_PCAP_ADJ_BIT_ISR_SOFT_RESETI },
	{ "SSP_PCAP_ADJ_BIT_ISR_MNEXBI", SSP_PCAP_ADJ_BIT_ISR_MNEXBI },

	{ "SSP_PCAP_ADJ_BIT_MSR_ADCDONEM", 	SSP_PCAP_ADJ_BIT_MSR_ADCDONEM },
	{ "SSP_PCAP_ADJ_BIT_MSR_TSM", SSP_PCAP_ADJ_BIT_MSR_TSM },
	{ "SSP_PCAP_ADJ_BIT_MSR_1HZM", SSP_PCAP_ADJ_BIT_MSR_1HZM },
	{ "SSP_PCAP_ADJ_BIT_MSR_WHM", SSP_PCAP_ADJ_BIT_MSR_WHM },
	{ "SSP_PCAP_ADJ_BIT_MSR_WLM", SSP_PCAP_ADJ_BIT_MSR_WLM },
	{ "SSP_PCAP_ADJ_BIT_MSR_TODAM", SSP_PCAP_ADJ_BIT_MSR_TODAM },
	{ "SSP_PCAP_ADJ_BIT_MSR_USB4VM", SSP_PCAP_ADJ_BIT_MSR_USB4VM },
	{ "SSP_PCAP_ADJ_BIT_MSR_ONOFFM", SSP_PCAP_ADJ_BIT_MSR_ONOFFM },
	{ "SSP_PCAP_ADJ_BIT_MSR_ONOFF2M", SSP_PCAP_ADJ_BIT_MSR_ONOFF2M },
	{ "SSP_PCAP_ADJ_BIT_MSR_USB1VM", SSP_PCAP_ADJ_BIT_MSR_USB1VM },
	{ "SSP_PCAP_ADJ_BIT_MSR_MOBPORTM", SSP_PCAP_ADJ_BIT_MSR_MOBPORTM },
	{ "SSP_PCAP_ADJ_BIT_MSR_MB2M", SSP_PCAP_ADJ_BIT_MSR_MB2M },
	{ "SSP_PCAP_ADJ_BIT_MSR_A1M", SSP_PCAP_ADJ_BIT_MSR_A1M },
	{ "SSP_PCAP_ADJ_BIT_MSR_STM", SSP_PCAP_ADJ_BIT_MSR_STM },
	{ "SSP_PCAP_ADJ_BIT_MSR_PCM", SSP_PCAP_ADJ_BIT_MSR_PCM },
	{ "SSP_PCAP_ADJ_BIT_MSR_WARMM", SSP_PCAP_ADJ_BIT_MSR_WARMM },
	{ "SSP_PCAP_ADJ_BIT_MSR_EOLM", SSP_PCAP_ADJ_BIT_MSR_EOLM },
	{ "SSP_PCAP_ADJ_BIT_MSR_CLKM", SSP_PCAP_ADJ_BIT_MSR_CLKM },
	{ "SSP_PCAP_ADJ_BIT_MSR_SYS_RSTM", SSP_PCAP_ADJ_BIT_MSR_SYS_RSTM },
	{ "SSP_PCAP_ADJ_BIT_MSR_ADCDONE2M", SSP_PCAP_ADJ_BIT_MSR_ADCDONE2M },
	{ "SSP_PCAP_ADJ_BIT_MSR_SOFT_RESETM", SSP_PCAP_ADJ_BIT_MSR_SOFT_RESETM },
	{ "SSP_PCAP_ADJ_BIT_MSR_MNEXBM", SSP_PCAP_ADJ_BIT_MSR_MNEXBM },

	{ "SSP_PCAP_ADJ_BIT_PSTAT_USBDET_4V", 	SSP_PCAP_ADJ_BIT_PSTAT_USBDET_4V },
	{ "SSP_PCAP_ADJ_BIT_PSTAT_ONOFFSNS", SSP_PCAP_ADJ_BIT_PSTAT_ONOFFSNS },
	{ "SSP_PCAP_ADJ_BIT_PSTAT_ONOFFSNS2", SSP_PCAP_ADJ_BIT_PSTAT_ONOFFSNS2 },
	{ "SSP_PCAP_ADJ_BIT_PSTAT_USBDET_1V", SSP_PCAP_ADJ_BIT_PSTAT_USBDET_1V },
	{ "SSP_PCAP_ADJ_BIT_PSTAT_MOBSENSB", SSP_PCAP_ADJ_BIT_PSTAT_MOBSENSB },
	{ "SSP_PCAP_ADJ_BIT_PSTAT_MB2SNS", SSP_PCAP_ADJ_BIT_PSTAT_MB2SNS },
	{ "SSP_PCAP_ADJ_BIT_PSTAT_A1SNS", SSP_PCAP_ADJ_BIT_PSTAT_A1SNS },
	{ "SSP_PCAP_ADJ_BIT_PSTAT_MSTB", SSP_PCAP_ADJ_BIT_PSTAT_MSTB },
	{ "SSP_PCAP_ADJ_BIT_PSTAT_EOL_STAT", SSP_PCAP_ADJ_BIT_PSTAT_EOL_STAT },
	{ "SSP_PCAP_ADJ_BIT_PSTAT_CLK_STAT", SSP_PCAP_ADJ_BIT_PSTAT_CLK_STAT },
	{ "SSP_PCAP_ADJ_BIT_PSTAT_SYS_RST", SSP_PCAP_ADJ_BIT_PSTAT_SYS_RST },
	{ "SSP_PCAP_ADJ_BIT_PSTAT_BATTFBSNS", SSP_PCAP_ADJ_BIT_PSTAT_BATTFBSNS },
	{ "SSP_PCAP_ADJ_BIT_PSTAT_BATT_DET_IN_SNS", SSP_PCAP_ADJ_BIT_PSTAT_BATT_DET_IN_SNS },
	{ "SSP_PCAP_ADJ_BIT_PSTAT_MNEXBSNS", SSP_PCAP_ADJ_BIT_PSTAT_MNEXBSNS },
	{ "SSP_PCAP_ADJ_BIT_PSTAT_WARM_SYS_RST", SSP_PCAP_ADJ_BIT_PSTAT_WARM_SYS_RST },

	{ "SSP_PCAP_ADJ_BIT_VREG2_V1_STBY", 	SSP_PCAP_ADJ_BIT_VREG2_V1_STBY },
	{ "SSP_PCAP_ADJ_BIT_VREG2_V2_STBY", SSP_PCAP_ADJ_BIT_VREG2_V2_STBY },
	{ "SSP_PCAP_ADJ_BIT_VREG2_V3_STBY", SSP_PCAP_ADJ_BIT_VREG2_V3_STBY },
	{ "SSP_PCAP_ADJ_BIT_VREG2_V4_STBY", SSP_PCAP_ADJ_BIT_VREG2_V4_STBY },
	{ "SSP_PCAP_ADJ_BIT_VREG2_V5_STBY", SSP_PCAP_ADJ_BIT_VREG2_V5_STBY },
	{ "SSP_PCAP_ADJ_BIT_VREG2_V6_STBY", SSP_PCAP_ADJ_BIT_VREG2_V6_STBY },
	{ "SSP_PCAP_ADJ_BIT_VREG2_V7_STBY", SSP_PCAP_ADJ_BIT_VREG2_V7_STBY },
	{ "SSP_PCAP_ADJ_BIT_VREG2_V8_STBY", SSP_PCAP_ADJ_BIT_VREG2_V8_STBY },
	{ "SSP_PCAP_ADJ_BIT_VREG2_V9_STBY", SSP_PCAP_ADJ_BIT_VREG2_V9_STBY },
	{ "SSP_PCAP_ADJ_BIT_VREG2_V10_STBY", SSP_PCAP_ADJ_BIT_VREG2_V10_STBY },
	{ "SSP_PCAP_ADJ_BIT_VREG2_V1_LOWPWR", SSP_PCAP_ADJ_BIT_VREG2_V1_LOWPWR },
	{ "SSP_PCAP_ADJ_BIT_VREG2_V2_LOWPWR", SSP_PCAP_ADJ_BIT_VREG2_V2_LOWPWR },
	{ "SSP_PCAP_ADJ_BIT_VREG2_V3_LOWPWR", SSP_PCAP_ADJ_BIT_VREG2_V3_LOWPWR },
	{ "SSP_PCAP_ADJ_BIT_VREG2_V4_LOWPWR", SSP_PCAP_ADJ_BIT_VREG2_V4_LOWPWR },
	{ "SSP_PCAP_ADJ_BIT_VREG2_V5_LOWPWR", SSP_PCAP_ADJ_BIT_VREG2_V5_LOWPWR },
	{ "SSP_PCAP_ADJ_BIT_VREG2_V6_LOWPWR", SSP_PCAP_ADJ_BIT_VREG2_V6_LOWPWR },
	{ "SSP_PCAP_ADJ_BIT_VREG2_V7_LOWPWR", SSP_PCAP_ADJ_BIT_VREG2_V7_LOWPWR },
	{ "SSP_PCAP_ADJ_BIT_VREG2_V8_LOWPWR", SSP_PCAP_ADJ_BIT_VREG2_V8_LOWPWR },
	{ "SSP_PCAP_ADJ_BIT_VREG2_V9_LOWPWR", SSP_PCAP_ADJ_BIT_VREG2_V9_LOWPWR },
	{ "SSP_PCAP_ADJ_BIT_VREG2_V10_LOWPWR", SSP_PCAP_ADJ_BIT_VREG2_V10_LOWPWR },

	{ "SSP_PCAP_ADJ_BIT_AUX_VREG_VAUX1_EN", SSP_PCAP_ADJ_BIT_AUX_VREG_VAUX1_EN },
	{ "SSP_PCAP_ADJ_BIT_AUX_VREG_VAUX1_0", SSP_PCAP_ADJ_BIT_AUX_VREG_VAUX1_0 },
	{ "SSP_PCAP_ADJ_BIT_AUX_VREG_VAUX1_1", SSP_PCAP_ADJ_BIT_AUX_VREG_VAUX1_1 },
	{ "SSP_PCAP_ADJ_BIT_AUX_VREG_VAUX2_EN", SSP_PCAP_ADJ_BIT_AUX_VREG_VAUX2_EN },
	{ "SSP_PCAP_ADJ_BIT_AUX_VREG_VAUX2_0", SSP_PCAP_ADJ_BIT_AUX_VREG_VAUX2_0 },
	{ "SSP_PCAP_ADJ_BIT_AUX_VREG_VAUX2_1", SSP_PCAP_ADJ_BIT_AUX_VREG_VAUX2_1 },
	{ "SSP_PCAP_ADJ_BIT_AUX_VREG_VAUX3_EN", SSP_PCAP_ADJ_BIT_AUX_VREG_VAUX3_EN },
	{ "SSP_PCAP_ADJ_BIT_AUX_VREG_VAUX3_0", SSP_PCAP_ADJ_BIT_AUX_VREG_VAUX3_0 },
	{ "SSP_PCAP_ADJ_BIT_AUX_VREG_VAUX3_1", SSP_PCAP_ADJ_BIT_AUX_VREG_VAUX3_1 },
	{ "SSP_PCAP_ADJ_BIT_AUX_VREG_VAUX3_2", SSP_PCAP_ADJ_BIT_AUX_VREG_VAUX3_2 },
	{ "SSP_PCAP_ADJ_BIT_AUX_VREG_VAUX3_3", SSP_PCAP_ADJ_BIT_AUX_VREG_VAUX3_3 },
	{ "SSP_PCAP_ADJ_BIT_AUX_VREG_VAUX4_EN", SSP_PCAP_ADJ_BIT_AUX_VREG_VAUX4_EN },
	{ "SSP_PCAP_ADJ_BIT_AUX_VREG_VAUX4_0", SSP_PCAP_ADJ_BIT_AUX_VREG_VAUX4_0 },
	{ "SSP_PCAP_ADJ_BIT_AUX_VREG_VAUX4_1", SSP_PCAP_ADJ_BIT_AUX_VREG_VAUX4_1 },
	{ "SSP_PCAP_ADJ_BIT_AUX_VREG_VSIM2_EN", SSP_PCAP_ADJ_BIT_AUX_VREG_VSIM2_EN },
	{ "SSP_PCAP_ADJ_BIT_AUX_VREG_VSIM_EN", SSP_PCAP_ADJ_BIT_AUX_VREG_VSIM_EN },
	{ "SSP_PCAP_ADJ_BIT_AUX_VREG_VSIM_0", SSP_PCAP_ADJ_BIT_AUX_VREG_VSIM_0 },
	{ "SSP_PCAP_ADJ_BIT_AUX_VREG_V_VIB_EN", SSP_PCAP_ADJ_BIT_AUX_VREG_V_VIB_EN },
	{ "SSP_PCAP_ADJ_BIT_AUX_VREG_V_VIB_0", SSP_PCAP_ADJ_BIT_AUX_VREG_V_VIB_0 },
	{ "SSP_PCAP_ADJ_BIT_AUX_VREG_V_VIB_1", SSP_PCAP_ADJ_BIT_AUX_VREG_V_VIB_1 },
	{ "SSP_PCAP_ADJ_BIT_AUX_VREG_VAUX1_STBY", SSP_PCAP_ADJ_BIT_AUX_VREG_VAUX1_STBY },
	{ "SSP_PCAP_ADJ_BIT_AUX_VREG_VAUX1_LOWPWR", SSP_PCAP_ADJ_BIT_AUX_VREG_VAUX1_LOWPWR },
	{ "SSP_PCAP_ADJ_BIT_AUX_VREG_SW3_STBY", SSP_PCAP_ADJ_BIT_AUX_VREG_SW3_STBY },

	{ "SSP_PCAP_ADJ_BIT_BATT_DAC_DAC0", 	SSP_PCAP_ADJ_BIT_BATT_DAC_DAC0 },
	{ "SSP_PCAP_ADJ_BIT_BATT_DAC_DAC1", SSP_PCAP_ADJ_BIT_BATT_DAC_DAC1 },
	{ "SSP_PCAP_ADJ_BIT_BATT_DAC_DAC2", SSP_PCAP_ADJ_BIT_BATT_DAC_DAC2 },
	{ "SSP_PCAP_ADJ_BIT_BATT_DAC_DAC3", SSP_PCAP_ADJ_BIT_BATT_DAC_DAC3 },
	{ "SSP_PCAP_ADJ_BIT_BATT_DAC_DAC4", SSP_PCAP_ADJ_BIT_BATT_DAC_DAC4 },
	{ "SSP_PCAP_ADJ_BIT_BATT_DAC_DAC5", SSP_PCAP_ADJ_BIT_BATT_DAC_DAC5 },
	{ "SSP_PCAP_ADJ_BIT_BATT_DAC_DAC6", SSP_PCAP_ADJ_BIT_BATT_DAC_DAC6 },
	{ "SSP_PCAP_ADJ_BIT_BATT_DAC_DAC7", SSP_PCAP_ADJ_BIT_BATT_DAC_DAC7 },
	{ "SSP_PCAP_ADJ_BIT_BATT_DAC_B_FDBK", SSP_PCAP_ADJ_BIT_BATT_DAC_B_FDBK },
	{ "SSP_PCAP_ADJ_BIT_BATT_DAC_EXT_ISENSE", SSP_PCAP_ADJ_BIT_BATT_DAC_EXT_ISENSE },
	{ "SSP_PCAP_ADJ_BIT_BATT_DAC_V_COIN0", SSP_PCAP_ADJ_BIT_BATT_DAC_V_COIN0 },
	{ "SSP_PCAP_ADJ_BIT_BATT_DAC_V_COIN1", SSP_PCAP_ADJ_BIT_BATT_DAC_V_COIN1 },
	{ "SSP_PCAP_ADJ_BIT_BATT_DAC_V_COIN2", SSP_PCAP_ADJ_BIT_BATT_DAC_V_COIN2 },
	{ "SSP_PCAP_ADJ_BIT_BATT_DAC_V_COIN3", SSP_PCAP_ADJ_BIT_BATT_DAC_V_COIN3 },
	{ "SSP_PCAP_ADJ_BIT_BATT_DAC_I_COIN", SSP_PCAP_ADJ_BIT_BATT_DAC_I_COIN },
	{ "SSP_PCAP_ADJ_BIT_BATT_DAC_COIN_CH_EN", SSP_PCAP_ADJ_BIT_BATT_DAC_COIN_CH_EN },
	{ "SSP_PCAP_ADJ_BIT_BATT_DAC_EOL_SEL0", SSP_PCAP_ADJ_BIT_BATT_DAC_EOL_SEL0 },
	{ "SSP_PCAP_ADJ_BIT_BATT_DAC_EOL_SEL1", SSP_PCAP_ADJ_BIT_BATT_DAC_EOL_SEL1 },
	{ "SSP_PCAP_ADJ_BIT_BATT_DAC_EOL_SEL2", SSP_PCAP_ADJ_BIT_BATT_DAC_EOL_SEL2 },
	{ "SSP_PCAP_ADJ_BIT_BATT_DAC_EOL_CMP_EN", SSP_PCAP_ADJ_BIT_BATT_DAC_EOL_CMP_EN },
	{ "SSP_PCAP_ADJ_BIT_BATT_DAC_BATT_DET_EN", SSP_PCAP_ADJ_BIT_BATT_DAC_BATT_DET_EN },
	{ "SSP_PCAP_ADJ_BIT_BATT_DAC_THERMBIAS_CTRL", SSP_PCAP_ADJ_BIT_BATT_DAC_THERMBIAS_CTRL },

	{ "SSP_PCAP_ADJ_BIT_ADC1_ADEN", 	SSP_PCAP_ADJ_BIT_ADC1_ADEN },
	{ "SSP_PCAP_ADJ_BIT_ADC1_RAND", SSP_PCAP_ADJ_BIT_ADC1_RAND },
	{ "SSP_PCAP_ADJ_BIT_ADC1_AD_SEL1", SSP_PCAP_ADJ_BIT_ADC1_AD_SEL1 },
	{ "SSP_PCAP_ADJ_BIT_ADC1_AD_SEL2", SSP_PCAP_ADJ_BIT_ADC1_AD_SEL2 },
	{ "SSP_PCAP_ADJ_BIT_ADC1_ADA10", SSP_PCAP_ADJ_BIT_ADC1_ADA10 },
	{ "SSP_PCAP_ADJ_BIT_ADC1_ADA11", SSP_PCAP_ADJ_BIT_ADC1_ADA11 },
	{ "SSP_PCAP_ADJ_BIT_ADC1_ADA12", SSP_PCAP_ADJ_BIT_ADC1_ADA12 },
	{ "SSP_PCAP_ADJ_BIT_ADC1_ADA20", SSP_PCAP_ADJ_BIT_ADC1_ADA20 },
	{ "SSP_PCAP_ADJ_BIT_ADC1_ADA21", SSP_PCAP_ADJ_BIT_ADC1_ADA21 },
	{ "SSP_PCAP_ADJ_BIT_ADC1_ADA22", SSP_PCAP_ADJ_BIT_ADC1_ADA22 },
	{ "SSP_PCAP_ADJ_BIT_ADC1_ATO0", SSP_PCAP_ADJ_BIT_ADC1_ATO0 },
	{ "SSP_PCAP_ADJ_BIT_ADC1_ATO1", SSP_PCAP_ADJ_BIT_ADC1_ATO1 },
	{ "SSP_PCAP_ADJ_BIT_ADC1_ATO2", SSP_PCAP_ADJ_BIT_ADC1_ATO2 },
	{ "SSP_PCAP_ADJ_BIT_ADC1_ATO3", SSP_PCAP_ADJ_BIT_ADC1_ATO3 },
	{ "SSP_PCAP_ADJ_BIT_ADC1_ATOX", SSP_PCAP_ADJ_BIT_ADC1_ATOX },
	{ "SSP_PCAP_ADJ_BIT_ADC1_MTR1", SSP_PCAP_ADJ_BIT_ADC1_MTR1 },
	{ "SSP_PCAP_ADJ_BIT_ADC1_MTR2", SSP_PCAP_ADJ_BIT_ADC1_MTR2 },
	{ "SSP_PCAP_ADJ_BIT_ADC1_TS_M0", SSP_PCAP_ADJ_BIT_ADC1_TS_M0 },
	{ "SSP_PCAP_ADJ_BIT_ADC1_TS_M1", SSP_PCAP_ADJ_BIT_ADC1_TS_M1 },
	{ "SSP_PCAP_ADJ_BIT_ADC1_TS_M2", SSP_PCAP_ADJ_BIT_ADC1_TS_M2 },
	{ "SSP_PCAP_ADJ_BIT_ADC1_TS_REF_LOWPWR", SSP_PCAP_ADJ_BIT_ADC1_TS_REF_LOWPWR },
	{ "SSP_PCAP_ADJ_BIT_ADC1_TS_REFENB", SSP_PCAP_ADJ_BIT_ADC1_TS_REFENB },
	{ "SSP_PCAP_ADJ_BIT_ADC1_BATT_I_POLARITY", SSP_PCAP_ADJ_BIT_ADC1_BATT_I_POLARITY },
	{ "SSP_PCAP_ADJ_BIT_ADC1_BATT_I_ADC", SSP_PCAP_ADJ_BIT_ADC1_BATT_I_ADC },

	{ "SSP_PCAP_ADJ_BIT_ADC2_ADD10", SSP_PCAP_ADJ_BIT_ADC2_ADD10 },
	{ "SSP_PCAP_ADJ_BIT_ADC2_ADD11", SSP_PCAP_ADJ_BIT_ADC2_ADD11 },
	{ "SSP_PCAP_ADJ_BIT_ADC2_ADD12", SSP_PCAP_ADJ_BIT_ADC2_ADD12 },
	{ "SSP_PCAP_ADJ_BIT_ADC2_ADD13", SSP_PCAP_ADJ_BIT_ADC2_ADD13 },
	{ "SSP_PCAP_ADJ_BIT_ADC2_ADD14", SSP_PCAP_ADJ_BIT_ADC2_ADD14 },
	{ "SSP_PCAP_ADJ_BIT_ADC2_ADD15", SSP_PCAP_ADJ_BIT_ADC2_ADD15 },
	{ "SSP_PCAP_ADJ_BIT_ADC2_ADD16", SSP_PCAP_ADJ_BIT_ADC2_ADD16 },
	{ "SSP_PCAP_ADJ_BIT_ADC2_ADD17", SSP_PCAP_ADJ_BIT_ADC2_ADD17 },
	{ "SSP_PCAP_ADJ_BIT_ADC2_ADD18", SSP_PCAP_ADJ_BIT_ADC2_ADD18 },
	{ "SSP_PCAP_ADJ_BIT_ADC2_ADD19", SSP_PCAP_ADJ_BIT_ADC2_ADD19 },
	{ "SSP_PCAP_ADJ_BIT_ADC2_ADD20", SSP_PCAP_ADJ_BIT_ADC2_ADD20 },
	{ "SSP_PCAP_ADJ_BIT_ADC2_ADD21", SSP_PCAP_ADJ_BIT_ADC2_ADD21 },
	{ "SSP_PCAP_ADJ_BIT_ADC2_ADD22", SSP_PCAP_ADJ_BIT_ADC2_ADD22 },
	{ "SSP_PCAP_ADJ_BIT_ADC2_ADD23", SSP_PCAP_ADJ_BIT_ADC2_ADD23 },
	{ "SSP_PCAP_ADJ_BIT_ADC2_ADD24", SSP_PCAP_ADJ_BIT_ADC2_ADD24 },
	{ "SSP_PCAP_ADJ_BIT_ADC2_ADD25", SSP_PCAP_ADJ_BIT_ADC2_ADD25 },
	{ "SSP_PCAP_ADJ_BIT_ADC2_ADD26", SSP_PCAP_ADJ_BIT_ADC2_ADD26 },
	{ "SSP_PCAP_ADJ_BIT_ADC2_ADD27", SSP_PCAP_ADJ_BIT_ADC2_ADD27 },
	{ "SSP_PCAP_ADJ_BIT_ADC2_ADD28", SSP_PCAP_ADJ_BIT_ADC2_ADD28 },
	{ "SSP_PCAP_ADJ_BIT_ADC2_ADD29", SSP_PCAP_ADJ_BIT_ADC2_ADD29 },
	{ "SSP_PCAP_ADJ_BIT_ADC2_ADINC1", SSP_PCAP_ADJ_BIT_ADC2_ADINC1 },
	{ "SSP_PCAP_ADJ_BIT_ADC2_ADINC2", SSP_PCAP_ADJ_BIT_ADC2_ADINC2 },
	{ "SSP_PCAP_ADJ_BIT_ADC2_ASC", SSP_PCAP_ADJ_BIT_ADC2_ASC },

	{ "SSP_PCAP_ADJ_BIT_AUD_CODEC_AUDIHPF", 	SSP_PCAP_ADJ_BIT_AUD_CODEC_AUDIHPF },
	{ "SSP_PCAP_ADJ_BIT_AUD_CODEC_SMB", SSP_PCAP_ADJ_BIT_AUD_CODEC_SMB },
	{ "SSP_PCAP_ADJ_BIT_AUD_CODEC_AUDOHPF", SSP_PCAP_ADJ_BIT_AUD_CODEC_AUDOHPF },
	{ "SSP_PCAP_ADJ_BIT_AUD_CODEC_CD_TS", SSP_PCAP_ADJ_BIT_AUD_CODEC_CD_TS },
	{ "SSP_PCAP_ADJ_BIT_AUD_CODEC_DLM", SSP_PCAP_ADJ_BIT_AUD_CODEC_DLM },
	{ "SSP_PCAP_ADJ_BIT_AUD_CODEC_ADITH", SSP_PCAP_ADJ_BIT_AUD_CODEC_ADITH },
	{ "SSP_PCAP_ADJ_BIT_AUD_CODEC_CDC_CLK0", SSP_PCAP_ADJ_BIT_AUD_CODEC_CDC_CLK0 },
	{ "SSP_PCAP_ADJ_BIT_AUD_CODEC_CDC_CLK1", SSP_PCAP_ADJ_BIT_AUD_CODEC_CDC_CLK1 },
	{ "SSP_PCAP_ADJ_BIT_AUD_CODEC_CDC_CLK2", SSP_PCAP_ADJ_BIT_AUD_CODEC_CDC_CLK2 },
	{ "SSP_PCAP_ADJ_BIT_AUD_CODEC_CLK_INV", SSP_PCAP_ADJ_BIT_AUD_CODEC_CLK_INV },
	{ "SSP_PCAP_ADJ_BIT_AUD_CODEC_FS_INV", SSP_PCAP_ADJ_BIT_AUD_CODEC_FS_INV },
	{ "SSP_PCAP_ADJ_BIT_AUD_CODEC_DF_RESET", SSP_PCAP_ADJ_BIT_AUD_CODEC_DF_RESET },
	{ "SSP_PCAP_ADJ_BIT_AUD_CODEC_CDC_EN", SSP_PCAP_ADJ_BIT_AUD_CODEC_CDC_EN },
	{ "SSP_PCAP_ADJ_BIT_AUD_CODEC_CDC_CLK_EN", SSP_PCAP_ADJ_BIT_AUD_CODEC_CDC_CLK_EN },
	{ "SSP_PCAP_ADJ_BIT_AUD_CODEC_FS_8K_16K", SSP_PCAP_ADJ_BIT_AUD_CODEC_FS_8K_16K },
	{ "SSP_PCAP_ADJ_BIT_AUD_CODEC_DIG_AUD_IN", SSP_PCAP_ADJ_BIT_AUD_CODEC_DIG_AUD_IN },
	{ "SSP_PCAP_ADJ_BIT_AUD_CODEC_CLK_IN_SEL", SSP_PCAP_ADJ_BIT_AUD_CODEC_CLK_IN_SEL },
	{ "SSP_PCAP_ADJ_BIT_AUD_CODEC_MIC2_MUX", SSP_PCAP_ADJ_BIT_AUD_CODEC_MIC2_MUX },
	{ "SSP_PCAP_ADJ_BIT_AUD_CODEC_MIC2IG0", SSP_PCAP_ADJ_BIT_AUD_CODEC_MIC2IG0 },
	{ "SSP_PCAP_ADJ_BIT_AUD_CODEC_MIC2IG1", SSP_PCAP_ADJ_BIT_AUD_CODEC_MIC2IG1 },
	{ "SSP_PCAP_ADJ_BIT_AUD_CODEC_MIC2IG2", SSP_PCAP_ADJ_BIT_AUD_CODEC_MIC2IG2 },
	{ "SSP_PCAP_ADJ_BIT_AUD_CODEC_MIC2IG3", SSP_PCAP_ADJ_BIT_AUD_CODEC_MIC2IG3 },
	{ "SSP_PCAP_ADJ_BIT_AUD_CODEC_MIC2IG4", SSP_PCAP_ADJ_BIT_AUD_CODEC_MIC2IG4 },
	{ "SSP_PCAP_ADJ_BIT_AUD_CODEC_MIC2IG_PRI_ADJ", SSP_PCAP_ADJ_BIT_AUD_CODEC_MIC2IG_PRI_ADJ },
	{ "SSP_PCAP_ADJ_BIT_AUD_CODEC_CDC_PRI_ADJ", SSP_PCAP_ADJ_BIT_AUD_CODEC_CDC_PRI_ADJ },

	{ "SSP_PCAP_ADJ_BIT_AUD_RX_AMPS_A1_EN", 	SSP_PCAP_ADJ_BIT_AUD_RX_AMPS_A1_EN },
	{ "SSP_PCAP_ADJ_BIT_AUD_RX_AMPS_A2_EN", SSP_PCAP_ADJ_BIT_AUD_RX_AMPS_A2_EN },
	{ "SSP_PCAP_ADJ_BIT_AUD_RX_AMPS_A4_EN", SSP_PCAP_ADJ_BIT_AUD_RX_AMPS_A4_EN },
	{ "SSP_PCAP_ADJ_BIT_AUD_RX_AMPS_ARIGHT_EN", SSP_PCAP_ADJ_BIT_AUD_RX_AMPS_ARIGHT_EN },
	{ "SSP_PCAP_ADJ_BIT_AUD_RX_AMPS_ALEFT_EN", SSP_PCAP_ADJ_BIT_AUD_RX_AMPS_ALEFT_EN },
	{ "SSP_PCAP_ADJ_BIT_AUD_RX_AMPS_CD_BYP", SSP_PCAP_ADJ_BIT_AUD_RX_AMPS_CD_BYP },
	{ "SSP_PCAP_ADJ_BIT_AUD_RX_AMPS_CDC_SW", SSP_PCAP_ADJ_BIT_AUD_RX_AMPS_CDC_SW },
	{ "SSP_PCAP_ADJ_BIT_AUD_RX_AMPS_ST_DAC_SW", SSP_PCAP_ADJ_BIT_AUD_RX_AMPS_ST_DAC_SW },
	{ "SSP_PCAP_ADJ_BIT_AUD_RX_AMPS_PGA_IN_SW", SSP_PCAP_ADJ_BIT_AUD_RX_AMPS_PGA_IN_SW },
	{ "SSP_PCAP_ADJ_BIT_AUD_RX_AMPS_PGA_R_EN", SSP_PCAP_ADJ_BIT_AUD_RX_AMPS_PGA_R_EN },
	{ "SSP_PCAP_ADJ_BIT_AUD_RX_AMPS_PGA_L_EN", SSP_PCAP_ADJ_BIT_AUD_RX_AMPS_PGA_L_EN },
	{ "SSP_PCAP_ADJ_BIT_AUD_RX_AMPS_AUDOG0", SSP_PCAP_ADJ_BIT_AUD_RX_AMPS_AUDOG0 },
	{ "SSP_PCAP_ADJ_BIT_AUD_RX_AMPS_AUDOG1", SSP_PCAP_ADJ_BIT_AUD_RX_AMPS_AUDOG1 },
	{ "SSP_PCAP_ADJ_BIT_AUD_RX_AMPS_AUDOG2", SSP_PCAP_ADJ_BIT_AUD_RX_AMPS_AUDOG2 },
	{ "SSP_PCAP_ADJ_BIT_AUD_RX_AMPS_AUDOG3", SSP_PCAP_ADJ_BIT_AUD_RX_AMPS_AUDOG3 },
	{ "SSP_PCAP_ADJ_BIT_AUD_RX_AMPS_A1CTRL", SSP_PCAP_ADJ_BIT_AUD_RX_AMPS_A1CTRL },
	{ "SSP_PCAP_ADJ_BIT_AUD_RX_AMPS_MONO0", SSP_PCAP_ADJ_BIT_AUD_RX_AMPS_MONO0 },
	{ "SSP_PCAP_ADJ_BIT_AUD_RX_AMPS_MONO1", SSP_PCAP_ADJ_BIT_AUD_RX_AMPS_MONO1 },
	{ "SSP_PCAP_ADJ_BIT_AUD_RX_AMPS_AUDOG_PRI_ADJ", SSP_PCAP_ADJ_BIT_AUD_RX_AMPS_AUDOG_PRI_ADJ },
	{ "SSP_PCAP_ADJ_BIT_AUD_RX_AMPS_MONO_PRI_ADJ", SSP_PCAP_ADJ_BIT_AUD_RX_AMPS_MONO_PRI_ADJ },
	{ "SSP_PCAP_ADJ_BIT_AUD_RX_AMPS_RX_PRI_ADJ0", SSP_PCAP_ADJ_BIT_AUD_RX_AMPS_RX_PRI_ADJ0 },
	{ "SSP_PCAP_ADJ_BIT_AUD_RX_AMPS_RX_PRI_ADJ1", SSP_PCAP_ADJ_BIT_AUD_RX_AMPS_RX_PRI_ADJ1 },

	{ "SSP_PCAP_ADJ_BIT_ST_DAC_SMB_ST_DAC", 	SSP_PCAP_ADJ_BIT_ST_DAC_SMB_ST_DAC },
	{ "SSP_PCAP_ADJ_BIT_ST_DAC_STDET_EN", SSP_PCAP_ADJ_BIT_ST_DAC_STDET_EN },
	{ "SSP_PCAP_ADJ_BIT_ST_DAC_ST_CLK0", SSP_PCAP_ADJ_BIT_ST_DAC_ST_CLK0 },
	{ "SSP_PCAP_ADJ_BIT_ST_DAC_ST_CLK1", SSP_PCAP_ADJ_BIT_ST_DAC_ST_CLK1 },
	{ "SSP_PCAP_ADJ_BIT_ST_DAC_ST_CLK2", SSP_PCAP_ADJ_BIT_ST_DAC_ST_CLK2 },
	{ "SSP_PCAP_ADJ_BIT_ST_DAC_ST_CLK_EN", SSP_PCAP_ADJ_BIT_ST_DAC_ST_CLK_EN },
	{ "SSP_PCAP_ADJ_BIT_ST_DAC_DF_RESET_ST_DAC", SSP_PCAP_ADJ_BIT_ST_DAC_DF_RESET_ST_DAC },
	{ "SSP_PCAP_ADJ_BIT_ST_DAC_ST_DAC_EN", SSP_PCAP_ADJ_BIT_ST_DAC_ST_DAC_EN },
	{ "SSP_PCAP_ADJ_BIT_ST_DAC_SR0", SSP_PCAP_ADJ_BIT_ST_DAC_SR0 },
	{ "SSP_PCAP_ADJ_BIT_ST_DAC_SR1", SSP_PCAP_ADJ_BIT_ST_DAC_SR1 },
	{ "SSP_PCAP_ADJ_BIT_ST_DAC_SR2", SSP_PCAP_ADJ_BIT_ST_DAC_SR2 },
	{ "SSP_PCAP_ADJ_BIT_ST_DAC_SR3", SSP_PCAP_ADJ_BIT_ST_DAC_SR3 },
	{ "SSP_PCAP_ADJ_BIT_ST_DAC_DIG_AUD_IN_ST_DAC", SSP_PCAP_ADJ_BIT_ST_DAC_DIG_AUD_IN_ST_DAC },
	{ "SSP_PCAP_ADJ_BIT_ST_DAC_DIG_AUD_FS0", SSP_PCAP_ADJ_BIT_ST_DAC_DIG_AUD_FS0 },
	{ "SSP_PCAP_ADJ_BIT_ST_DAC_DIG_AUD_FS1", SSP_PCAP_ADJ_BIT_ST_DAC_DIG_AUD_FS1 },
	{ "SSP_PCAP_ADJ_BIT_ST_DAC_BCLK0", SSP_PCAP_ADJ_BIT_ST_DAC_BCLK0 },
	{ "SSP_PCAP_ADJ_BIT_ST_DAC_BCLK1", SSP_PCAP_ADJ_BIT_ST_DAC_BCLK1 },
	{ "SSP_PCAP_ADJ_BIT_ST_DAC_ST_CLK_INV", SSP_PCAP_ADJ_BIT_ST_DAC_ST_CLK_INV },
	{ "SSP_PCAP_ADJ_BIT_ST_DAC_ST_FS_INV", SSP_PCAP_ADJ_BIT_ST_DAC_ST_FS_INV },
	{ "SSP_PCAP_ADJ_BIT_ST_DAC_ST_DAC_CLK_IN_SEL", SSP_PCAP_ADJ_BIT_ST_DAC_ST_DAC_CLK_IN_SEL },
	{ "SSP_PCAP_ADJ_BIT_ST_DAC_ST_DAC_PRI_ADJ", SSP_PCAP_ADJ_BIT_ST_DAC_ST_DAC_PRI_ADJ },

	{ "SSP_PCAP_ADJ_BIT_BUSCTRL_FSENB", 	SSP_PCAP_ADJ_BIT_BUSCTRL_FSENB },
	{ "SSP_PCAP_ADJ_BIT_BUSCTRL_USB_SUSPEND", SSP_PCAP_ADJ_BIT_BUSCTRL_USB_SUSPEND },
	{ "SSP_PCAP_ADJ_BIT_BUSCTRL_USB_PU", SSP_PCAP_ADJ_BIT_BUSCTRL_USB_PU },
	{ "SSP_PCAP_ADJ_BIT_BUSCTRL_USB_PD", SSP_PCAP_ADJ_BIT_BUSCTRL_USB_PD },
	{ "SSP_PCAP_ADJ_BIT_BUSCTRL_VUSB_EN", SSP_PCAP_ADJ_BIT_BUSCTRL_VUSB_EN },
	{ "SSP_PCAP_ADJ_BIT_BUSCTRL_USB_PS", SSP_PCAP_ADJ_BIT_BUSCTRL_USB_PS },
	{ "SSP_PCAP_ADJ_BIT_BUSCTRL_VUSB_MSTR_EN", SSP_PCAP_ADJ_BIT_BUSCTRL_VUSB_MSTR_EN },
	{ "SSP_PCAP_ADJ_BIT_BUSCTRL_VBUS_PD_ENB", SSP_PCAP_ADJ_BIT_BUSCTRL_VBUS_PD_ENB },
	{ "SSP_PCAP_ADJ_BIT_BUSCTRL_CURRLIM", SSP_PCAP_ADJ_BIT_BUSCTRL_CURRLIM },
	{ "SSP_PCAP_ADJ_BIT_BUSCTRL_RS232ENB", SSP_PCAP_ADJ_BIT_BUSCTRL_RS232ENB },
	{ "SSP_PCAP_ADJ_BIT_BUSCTRL_RS232_DIR", SSP_PCAP_ADJ_BIT_BUSCTRL_RS232_DIR },
	{ "SSP_PCAP_ADJ_BIT_BUSCTRL_SE0_CONN", SSP_PCAP_ADJ_BIT_BUSCTRL_SE0_CONN },
	{ "SSP_PCAP_ADJ_BIT_BUSCTRL_USB_PDM", SSP_PCAP_ADJ_BIT_BUSCTRL_USB_PDM },
	{ "SSP_PCAP_ADJ_BIT_BUSCTRL_BUS_PRI_ADJ", SSP_PCAP_ADJ_BIT_BUSCTRL_BUS_PRI_ADJ },

	{ "SSP_PCAP_ADJ_BIT_PERIPH_BL_CTRL0", 	SSP_PCAP_ADJ_BIT_PERIPH_BL_CTRL0 },
	{ "SSP_PCAP_ADJ_BIT_PERIPH_BL_CTRL1", SSP_PCAP_ADJ_BIT_PERIPH_BL_CTRL1 },
	{ "SSP_PCAP_ADJ_BIT_PERIPH_BL_CTRL2", SSP_PCAP_ADJ_BIT_PERIPH_BL_CTRL2 },
	{ "SSP_PCAP_ADJ_BIT_PERIPH_BL_CTRL3", SSP_PCAP_ADJ_BIT_PERIPH_BL_CTRL3 },
	{ "SSP_PCAP_ADJ_BIT_PERIPH_BL_CTRL4", SSP_PCAP_ADJ_BIT_PERIPH_BL_CTRL4 },
	{ "SSP_PCAP_ADJ_BIT_PERIPH_LEDR_EN", SSP_PCAP_ADJ_BIT_PERIPH_LEDR_EN },
	{ "SSP_PCAP_ADJ_BIT_PERIPH_LEDG_EN", SSP_PCAP_ADJ_BIT_PERIPH_LEDG_EN },
	{ "SSP_PCAP_ADJ_BIT_PERIPH_LEDR_CTRL0", SSP_PCAP_ADJ_BIT_PERIPH_LEDR_CTRL0 },
	{ "SSP_PCAP_ADJ_BIT_PERIPH_LEDR_CTRL1", SSP_PCAP_ADJ_BIT_PERIPH_LEDR_CTRL1 },
	{ "SSP_PCAP_ADJ_BIT_PERIPH_LEDR_CTRL2", SSP_PCAP_ADJ_BIT_PERIPH_LEDR_CTRL2 },
	{ "SSP_PCAP_ADJ_BIT_PERIPH_LEDR_CTRL3", SSP_PCAP_ADJ_BIT_PERIPH_LEDR_CTRL3 },
	{ "SSP_PCAP_ADJ_BIT_PERIPH_LEDG_CTRL0", SSP_PCAP_ADJ_BIT_PERIPH_LEDG_CTRL0 },
	{ "SSP_PCAP_ADJ_BIT_PERIPH_LEDG_CTRL1", SSP_PCAP_ADJ_BIT_PERIPH_LEDG_CTRL1 },
	{ "SSP_PCAP_ADJ_BIT_PERIPH_LEDG_CTRL2", SSP_PCAP_ADJ_BIT_PERIPH_LEDG_CTRL2 },
	{ "SSP_PCAP_ADJ_BIT_PERIPH_LEDG_CTRL3", SSP_PCAP_ADJ_BIT_PERIPH_LEDG_CTRL3 },
	{ "SSP_PCAP_ADJ_BIT_PERIPH_LEDR_I0", SSP_PCAP_ADJ_BIT_PERIPH_LEDR_I0 },
	{ "SSP_PCAP_ADJ_BIT_PERIPH_LEDR_I1", SSP_PCAP_ADJ_BIT_PERIPH_LEDR_I1 },
	{ "SSP_PCAP_ADJ_BIT_PERIPH_LEDG_I0", SSP_PCAP_ADJ_BIT_PERIPH_LEDG_I0 },
	{ "SSP_PCAP_ADJ_BIT_PERIPH_LEDG_I1", SSP_PCAP_ADJ_BIT_PERIPH_LEDG_I1 },
	{ "SSP_PCAP_ADJ_BIT_PERIPH_SKIP", SSP_PCAP_ADJ_BIT_PERIPH_SKIP },
	{ "SSP_PCAP_ADJ_BIT_PERIPH_BL2_CTRL0", SSP_PCAP_ADJ_BIT_PERIPH_BL2_CTRL0 },
	{ "SSP_PCAP_ADJ_BIT_PERIPH_BL2_CTRL1", SSP_PCAP_ADJ_BIT_PERIPH_BL2_CTRL1 },
	{ "SSP_PCAP_ADJ_BIT_PERIPH_BL2_CTRL2", SSP_PCAP_ADJ_BIT_PERIPH_BL2_CTRL2 },
	{ "SSP_PCAP_ADJ_BIT_PERIPH_BL2_CTRL3", SSP_PCAP_ADJ_BIT_PERIPH_BL2_CTRL3 },
	{ "SSP_PCAP_ADJ_BIT_PERIPH_BL2_CTRL4", SSP_PCAP_ADJ_BIT_PERIPH_BL2_CTRL4 },

	{ "SSP_PCAP_ADJ_BIT_LOWPWR_CTRL_VAUX2_STBY", 	SSP_PCAP_ADJ_BIT_LOWPWR_CTRL_VAUX2_STBY },
	{ "SSP_PCAP_ADJ_BIT_LOWPWR_CTRL_VAUX2_LOWPWR", SSP_PCAP_ADJ_BIT_LOWPWR_CTRL_VAUX2_LOWPWR },
	{ "SSP_PCAP_ADJ_BIT_LOWPWR_CTRL_VAUX3_STBY", SSP_PCAP_ADJ_BIT_LOWPWR_CTRL_VAUX3_STBY },
	{ "SSP_PCAP_ADJ_BIT_LOWPWR_CTRL_VAUX3_LOWPWR", SSP_PCAP_ADJ_BIT_LOWPWR_CTRL_VAUX3_LOWPWR },
	{ "SSP_PCAP_ADJ_BIT_LOWPWR_CTRL_VAUX4_STBY", SSP_PCAP_ADJ_BIT_LOWPWR_CTRL_VAUX4_STBY },
	{ "SSP_PCAP_ADJ_BIT_LOWPWR_CTRL_VAUX4_LOWPWR", SSP_PCAP_ADJ_BIT_LOWPWR_CTRL_VAUX4_LOWPWR },
	{ "SSP_PCAP_ADJ_BIT_LOWPWR_CTRL_VSIM_LOWPWR", SSP_PCAP_ADJ_BIT_LOWPWR_CTRL_VSIM_LOWPWR },
	{ "SSP_PCAP_ADJ_BIT_LOWPWR_CTRL_VSIM2_LOWPWR", SSP_PCAP_ADJ_BIT_LOWPWR_CTRL_VSIM2_LOWPWR },
	{ "SSP_PCAP_ADJ_BIT_LOWPWR_CTRL_SW1_MODE00", SSP_PCAP_ADJ_BIT_LOWPWR_CTRL_SW1_MODE00 },
	{ "SSP_PCAP_ADJ_BIT_LOWPWR_CTRL_SW1_MODE01", SSP_PCAP_ADJ_BIT_LOWPWR_CTRL_SW1_MODE01 },
	{ "SSP_PCAP_ADJ_BIT_LOWPWR_CTRL_SW1_MODE10", SSP_PCAP_ADJ_BIT_LOWPWR_CTRL_SW1_MODE10 },
	{ "SSP_PCAP_ADJ_BIT_LOWPWR_CTRL_SW1_MODE11", SSP_PCAP_ADJ_BIT_LOWPWR_CTRL_SW1_MODE11 },
	{ "SSP_PCAP_ADJ_BIT_LOWPWR_CTRL_SW10_DVS", SSP_PCAP_ADJ_BIT_LOWPWR_CTRL_SW10_DVS },
	{ "SSP_PCAP_ADJ_BIT_LOWPWR_CTRL_SW11_DVS", SSP_PCAP_ADJ_BIT_LOWPWR_CTRL_SW11_DVS },
	{ "SSP_PCAP_ADJ_BIT_LOWPWR_CTRL_SW12_DVS", SSP_PCAP_ADJ_BIT_LOWPWR_CTRL_SW12_DVS },
	{ "SSP_PCAP_ADJ_BIT_LOWPWR_CTRL_SW13_DVS", SSP_PCAP_ADJ_BIT_LOWPWR_CTRL_SW13_DVS },
	{ "SSP_PCAP_ADJ_BIT_LOWPWR_CTRL_SW2_MODE00", SSP_PCAP_ADJ_BIT_LOWPWR_CTRL_SW2_MODE00 },
	{ "SSP_PCAP_ADJ_BIT_LOWPWR_CTRL_SW2_MODE01", SSP_PCAP_ADJ_BIT_LOWPWR_CTRL_SW2_MODE01 },
	{ "SSP_PCAP_ADJ_BIT_LOWPWR_CTRL_SW2_MODE10", SSP_PCAP_ADJ_BIT_LOWPWR_CTRL_SW2_MODE10 },
	{ "SSP_PCAP_ADJ_BIT_LOWPWR_CTRL_SW2_MODE11", SSP_PCAP_ADJ_BIT_LOWPWR_CTRL_SW2_MODE11 },
	{ "SSP_PCAP_ADJ_BIT_LOWPWR_CTRL_SW20_DVS", SSP_PCAP_ADJ_BIT_LOWPWR_CTRL_SW20_DVS },
	{ "SSP_PCAP_ADJ_BIT_LOWPWR_CTRL_SW21_DVS", SSP_PCAP_ADJ_BIT_LOWPWR_CTRL_SW21_DVS },
	{ "SSP_PCAP_ADJ_BIT_LOWPWR_CTRL_SW22_DVS", SSP_PCAP_ADJ_BIT_LOWPWR_CTRL_SW22_DVS },
	{ "SSP_PCAP_ADJ_BIT_LOWPWR_CTRL_SW23_DVS", SSP_PCAP_ADJ_BIT_LOWPWR_CTRL_SW23_DVS },
	{ "SSP_PCAP_ADJ_BIT_LOWPWR_CTRL_VC_STBY", SSP_PCAP_ADJ_BIT_LOWPWR_CTRL_VC_STBY },

	{ "SSP_PCAP_ADJ_BIT_TX_AUD_AMPS_AUDIG0", SSP_PCAP_ADJ_BIT_TX_AUD_AMPS_AUDIG0 },
	{ "SSP_PCAP_ADJ_BIT_TX_AUD_AMPS_AUDIG1", SSP_PCAP_ADJ_BIT_TX_AUD_AMPS_AUDIG1 },
	{ "SSP_PCAP_ADJ_BIT_TX_AUD_AMPS_AUDIG2", SSP_PCAP_ADJ_BIT_TX_AUD_AMPS_AUDIG2 },
	{ "SSP_PCAP_ADJ_BIT_TX_AUD_AMPS_AUDIG3", SSP_PCAP_ADJ_BIT_TX_AUD_AMPS_AUDIG3 },
	{ "SSP_PCAP_ADJ_BIT_TX_AUD_AMPS_AUDIG4", SSP_PCAP_ADJ_BIT_TX_AUD_AMPS_AUDIG4 },
	{ "SSP_PCAP_ADJ_BIT_TX_AUD_AMPS_A3_EN", SSP_PCAP_ADJ_BIT_TX_AUD_AMPS_A3_EN },
	{ "SSP_PCAP_ADJ_BIT_TX_AUD_AMPS_A3_MUX", SSP_PCAP_ADJ_BIT_TX_AUD_AMPS_A3_MUX },
	{ "SSP_PCAP_ADJ_BIT_TX_AUD_AMPS_A5_EN", SSP_PCAP_ADJ_BIT_TX_AUD_AMPS_A5_EN },
	{ "SSP_PCAP_ADJ_BIT_TX_AUD_AMPS_A5_MUX", SSP_PCAP_ADJ_BIT_TX_AUD_AMPS_A5_MUX },
	{ "SSP_PCAP_ADJ_BIT_TX_AUD_AMPS_EXT_MIC_MUX", SSP_PCAP_ADJ_BIT_TX_AUD_AMPS_EXT_MIC_MUX },
	{ "SSP_PCAP_ADJ_BIT_TX_AUD_AMPS_MB_ON2", SSP_PCAP_ADJ_BIT_TX_AUD_AMPS_MB_ON2 },
	{ "SSP_PCAP_ADJ_BIT_TX_AUD_AMPS_MB_ON1", SSP_PCAP_ADJ_BIT_TX_AUD_AMPS_MB_ON1 },
	{ "SSP_PCAP_ADJ_BIT_TX_AUD_AMPS_A1ID_TX", SSP_PCAP_ADJ_BIT_TX_AUD_AMPS_A1ID_TX },
	{ "SSP_PCAP_ADJ_BIT_TX_AUD_AMPS_A1_CONFIG", SSP_PCAP_ADJ_BIT_TX_AUD_AMPS_A1_CONFIG },
	{ "SSP_PCAP_ADJ_BIT_TX_AUD_AMPS_AHS_CONFIG", SSP_PCAP_ADJ_BIT_TX_AUD_AMPS_AHS_CONFIG },
	{ "SSP_PCAP_ADJ_BIT_TX_AUD_AMPS_A2_CONFIG", SSP_PCAP_ADJ_BIT_TX_AUD_AMPS_A2_CONFIG },
	{ "SSP_PCAP_ADJ_BIT_TX_AUD_AMPS_AUDIO_LOWPWR", SSP_PCAP_ADJ_BIT_TX_AUD_AMPS_AUDIO_LOWPWR },
	{ "SSP_PCAP_ADJ_BIT_TX_AUD_AMPS_AUDIO_STBY", SSP_PCAP_ADJ_BIT_TX_AUD_AMPS_AUDIO_STBY },
	{ "SSP_PCAP_ADJ_BIT_TX_AUD_AMPS_V2_EN_2", SSP_PCAP_ADJ_BIT_TX_AUD_AMPS_V2_EN_2 },
	{ "SSP_PCAP_ADJ_BIT_TX_AUD_AMPS_AUDIG_PRI_ADJ", SSP_PCAP_ADJ_BIT_TX_AUD_AMPS_AUDIG_PRI_ADJ },
	{ "SSP_PCAP_ADJ_BIT_TX_AUD_AMPS_TX_PRI_ADJ0", SSP_PCAP_ADJ_BIT_TX_AUD_AMPS_TX_PRI_ADJ0 },
	{ "SSP_PCAP_ADJ_BIT_TX_AUD_AMPS_TX_PRI_ADJ1", SSP_PCAP_ADJ_BIT_TX_AUD_AMPS_TX_PRI_ADJ1 },

	{ "SSP_PCAP_ADJ_BIT_SYS_RST_CLR", SSP_PCAP_ADJ_BIT_SYS_RST_CLR },
	{ "SSP_PCAP_ADJ_BIT_SYS_RST_MODE0", SSP_PCAP_ADJ_BIT_SYS_RST_MODE0 },
	{ "SSP_PCAP_ADJ_BIT_SYS_RST_MODE1", SSP_PCAP_ADJ_BIT_SYS_RST_MODE1 },
	{ "SSP_PCAP_ADJ_BIT_SYS_VFLASH_0", SSP_PCAP_ADJ_BIT_SYS_VFLASH_0 },
	{ "SSP_PCAP_ADJ_BIT_SYS_VFLASH_1", SSP_PCAP_ADJ_BIT_SYS_VFLASH_1 },
	{ "SSP_PCAP_ADJ_BIT_SYS_MID_SELECT", SSP_PCAP_ADJ_BIT_SYS_MID_SELECT },
	{ "SSP_PCAP_ADJ_BIT_SYS_MID_FET", SSP_PCAP_ADJ_BIT_SYS_MID_FET },
	{ "SSP_PCAP_ADJ_BIT_SYS_MAIN_LOW", SSP_PCAP_ADJ_BIT_SYS_MAIN_LOW },
	{ "SSP_PCAP_ADJ_BIT_SYS_BATTFB_DIS", SSP_PCAP_ADJ_BIT_SYS_BATTFB_DIS },
	{ "SSP_PCAP_ADJ_BIT_SYS_GP_REG9", SSP_PCAP_ADJ_BIT_SYS_GP_REG9 },
	{ "SSP_PCAP_ADJ_BIT_SYS_GP_REG10", SSP_PCAP_ADJ_BIT_SYS_GP_REG10 },
	{ "SSP_PCAP_ADJ_BIT_SYS_GP_REG11", SSP_PCAP_ADJ_BIT_SYS_GP_REG11 },
	{ "SSP_PCAP_ADJ_BIT_SYS_GP_REG12", SSP_PCAP_ADJ_BIT_SYS_GP_REG12 },
	{ "SSP_PCAP_ADJ_BIT_SYS_GP_REG13", SSP_PCAP_ADJ_BIT_SYS_GP_REG13 },
	{ "SSP_PCAP_ADJ_BIT_SYS_GP_REG14", SSP_PCAP_ADJ_BIT_SYS_GP_REG14 },
	{ "SSP_PCAP_ADJ_BIT_SYS_GP_REG15", SSP_PCAP_ADJ_BIT_SYS_GP_REG15 },
	{ "SSP_PCAP_ADJ_BIT_SYS_GP_REG16", SSP_PCAP_ADJ_BIT_SYS_GP_REG16 },
	{ "SSP_PCAP_ADJ_BIT_SYS_GP_REG17", SSP_PCAP_ADJ_BIT_SYS_GP_REG17 },
	{ "SSP_PCAP_ADJ_BIT_SYS_GP_REG18", SSP_PCAP_ADJ_BIT_SYS_GP_REG18 },
	{ "SSP_PCAP_ADJ_BIT_SYS_GP_REG19", SSP_PCAP_ADJ_BIT_SYS_GP_REG19 },
	{ "SSP_PCAP_ADJ_BIT_SYS_GP_REG20", SSP_PCAP_ADJ_BIT_SYS_GP_REG20 },
	{ "SSP_PCAP_ADJ_BIT_SYS_GP_REG21", SSP_PCAP_ADJ_BIT_SYS_GP_REG21 },
	{ "SSP_PCAP_ADJ_BIT_SYS_GP_REG22", SSP_PCAP_ADJ_BIT_SYS_GP_REG22 },
	{ "SSP_PCAP_ADJ_BIT_SYS_GP_REG23", SSP_PCAP_ADJ_BIT_SYS_GP_REG23 },
	{ "SSP_PCAP_ADJ_BIT_SYS_GP_REG24", SSP_PCAP_ADJ_BIT_SYS_GP_REG24 },

	{ 0, 0 },
};

static int __init init_pcap_proc(void) {
	struct proc_dir_entry *pcap_dir = proc_mkdir("pcap", 0);
	struct proc_dir_entry *e;
	struct pcap_reg *r = pcap_regs;

	if(*((int*)SSP_PCAP_bit_set) != 0xe1a0c00d) { // check for mov ip, sp
		printk("cannot find function prologue. most likely loading in wrong kernel!!\n");
		return -EINVAL;
	}

	if(*((int*)SSP_PCAP_bit_clean) != 0xe1a0c00d) { // check for mov ip, sp
		printk("cannot find function prologue. most likely loading in wrong kernel!!\n");
		return -EINVAL;
	}

	if(*((int*)SSP_PCAP_get_bit_from_buffer) != 0xe1a0c00d) { // check for mov ip, sp
		printk("cannot find function prologue. most likely loading in wrong kernel!!\n");
		return -EINVAL;
	}

	while(r->name) {
		if ((e = create_proc_entry(r->name, 0644, pcap_dir))) {
			e->read_proc = pcap_read_proc;
			e->write_proc = pcap_write_proc;
			e->data = r;
		}

		r++;
	}

	if ((e = create_proc_entry("usb-regs", 0444, 0))) {
		e->read_proc = usb_reg_read_proc;
	}

	return 0;
}

static void __exit exit_pcap_proc(void) {
	remove_proc_entry("pcap", 0);
	remove_proc_entry("usb-regs", 0);
}

static int pcap_read_proc(char *page, char **start, off_t offset, int len, int *eof, void *data) {
	struct pcap_reg *r = (struct pcap_reg*)data;
	return sprintf(page, "%d\n", SSP_PCAP_get_bit_from_buffer(r->regbit));
}

static int pcap_write_proc(struct file *file, const char *buffer, unsigned long count, void *data) {
	struct pcap_reg *r = (struct pcap_reg*)data;
	char bit;
	if(copy_from_user(&bit, buffer, 1)) {
		return -EFAULT;
	}

	if(bit == '1') {
		SSP_PCAP_bit_set(r->regbit);
	} else if(bit == '0') {
		SSP_PCAP_bit_clean(r->regbit);
	} else {
		return -EINVAL;
	}

	return count;
}

static int usb_reg_read_proc(char *page, char **start, off_t offset, int len, int *eof, void *data) {
	return sprintf(page, "UDCCR = %08x\nUDCICR = [%08x %08x]\nUDCISR = [%08x %08x]\nUDCFNR = %08x\nUDCCSR0 = %08x\nUDCOTGICR = %08x\nUP2OCR = %08x\nUP3OCR = %08x\n\nUHCRHDA = %08x\nUHCRHDB = %08x\nUHCRHS = %08x\nUHCRHPS1 = %08x\nUHCRHPS2 = %08x\nUHCRHPS3 = %08x\n", 
		UDCCR, 
		UDCICR0, UDCICR1, 
		UDCISR0, UDCISR1,
		UDCFNR, 
		UDCCSR0,
		UDCOTGICR,
		UP2OCR, UP3OCR,

		UHCRHDA, UHCRHDB, UHCRHS,
		UHCRHPS1, UHCRHPS2, UHCRHPS3
	);

}

MODULE_DESCRIPTION("PCAP procfs interface");
MODULE_LICENSE("GPL");

module_init(init_pcap_proc);
module_exit(exit_pcap_proc);
